package org.rcisoft.business.blabel.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.bcourse.dao.BCourseRepository;
import org.rcisoft.business.bcourse.dto.QueryCourseResDTO;
import org.rcisoft.business.blabel.dao.BLabelRepository;
import org.rcisoft.business.blabel.dto.HomeParamDTO;
import org.rcisoft.business.blabel.dto.QueryLabelResDTO;
import org.rcisoft.business.blabel.dto.SearchParamDTO;
import org.rcisoft.business.blabel.entity.BLabel;
import org.rcisoft.business.blabel.service.BLabelService;
import org.rcisoft.business.blesson.dao.BLessonRepository;
import org.rcisoft.business.blesson.entity.BLesson;
import org.rcisoft.business.blesson.service.impl.BLessonServiceImpl;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.ResultServiceEnums;
import org.rcisoft.core.util.LogUtil;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.sys.user.bean.CurUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

import static java.util.Arrays.asList;

@Service
@Slf4j
public class BLabelServiceImpl implements BLabelService {

    @Autowired
    private BLabelRepository bLabelRepository;
    @Autowired
    private BLessonServiceImpl bLessonService;
    @Autowired
    private BLessonRepository bLessonRepository;
    @Autowired
    BCourseRepository bCourseRepository;

    @Override
    public List<BLabel> queryBlabelsByPagination(PageUtil pageUtil, String lName, String corpId) {
        return bLabelRepository.queryBlabels(lName, corpId);
    }

    @Override
    public List<BLabel> queryBlabels(String lName, CurUser curUser) {
        List<BLabel> labels = bLabelRepository.queryBlabels(lName, curUser.getCorpId());
        List<BLabel> labelsForPlatform = bLabelRepository.queryBLabelsForPlatform(lName, curUser.getCorpId());
        labels.addAll(labelsForPlatform);
        List<BLesson> lessons = addPushLable(bLessonService.queryHomeBLessons(curUser),curUser);

        LogUtil.addnNewLog("-----------addNewLable----------首页所有课程总个数--"+lessons.size()+"--------------------");

        //筛选 new lesson
        List<BLesson> newLessons = new ArrayList<>();
        for(BLesson b : lessons){
            if("1".equals(b.getIsNew()))
                newLessons.add(b);
        }
        for (BLabel label : labels) {
            int num = 0;
            for (BLesson lesson : newLessons) {
                List<QueryLabelResDTO> labelResDTOS = lesson.getLabelList();
                if (labelResDTOS != null && labelResDTOS.size() > 0) {
                    for (QueryLabelResDTO dto : labelResDTOS) {
                        if (dto.getBusinessId().equals(label.getBusinessId()))
                            num++;
                    }
                }
            }
            LogUtil.addnNewLog("-----------addNewLable----------当前标签--"+label+"--的new课程个数--"+num+"--------------------");
            label.setNewCount(num);
        }
        return labels;
    }

    @Override
    public HomeParamDTO queryBlabelAndCourse(SearchParamDTO searchParamDTO, CurUser curUser) {
        //新建返回对象
        HomeParamDTO paramDTO = new HomeParamDTO();
        //根据条件查询出对应的课程
        List<BLesson> lessons = bLessonService.queryListAllLesson(searchParamDTO,curUser);
        //筛选 new lesson
        List<BLesson> newLessons = new ArrayList<>();
        for(BLesson b : lessons){
            if("1".equals(b.getIsNew()))
                newLessons.add(b);
        }
        paramDTO.setAllNewCount(newLessons.size());
        LogUtil.addnNewLog("-----------addNewLable----------当前查询课程总个数--"+lessons.size()+"--------------------");
        //遍历分类数据
        List<QueryCourseResDTO> dtoList =  bCourseRepository.queryFirstLevel(curUser.getCorpId(),20);
        paramDTO.setCourses(addNewFolder(dtoList,newLessons));
        //遍历标签数据
        List<BLabel> labels = bLabelRepository.queryBlabels(searchParamDTO.getLName(), curUser.getCorpId());
        List<BLabel> labelsForPlatform = bLabelRepository.queryBLabelsForPlatform(searchParamDTO.getLName(), curUser.getCorpId());
        labels.addAll(labelsForPlatform);
        for (BLabel label : labels) {
            int num = 0;
            for (BLesson lesson : newLessons) {
                List<QueryLabelResDTO> labelResDTOS = lesson.getLabelList();
                if (labelResDTOS != null && labelResDTOS.size() > 0) {
                    for (QueryLabelResDTO dto : labelResDTOS) {
                        if (dto.getBusinessId().equals(label.getBusinessId()))
                            num++;
                    }
                }
            }
            LogUtil.addnNewLog("-----------addNewLable----------当前标签--"+label+"--的new课程个数--"+num+"--------------------");
            label.setNewCount(num);
        }
        List<BLabel> newLables = new ArrayList<>();
        labels.forEach(lable->{
            if(curUser.getCorpId().equals(lable.getCorpId()) || (!curUser.getCorpId().equals(lable.getCorpId()) && 0 < lable.getNewCount()))
                newLables.add(lable);
        });
        paramDTO.setLabels(newLables);
        return paramDTO;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel insertLabel(BLabel bLabel, CurUser curUser) {
        if (StringUtils.isBlank(bLabel.getLName())) {
            throw new ServiceException("标签名称不能为空");
        }
        int line = bLabelRepository.checknameByName(bLabel.getLName(), bLabel.getBusinessId(), curUser.getCorpId());
        if (line > 0) {
            throw new ServiceException(ResultServiceEnums.NAME_IS_EXISTS);
        }
        UserUtil.setCurrentPersistOperation(bLabel);
        return new PersistModel(bLabelRepository.insert(bLabel), MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel updateLabel(BLabel bLabel, CurUser curUser) {
        if (StringUtils.isBlank(bLabel.getLName())) {
            throw new ServiceException("标签名称不能为空");
        }
        int line = bLabelRepository.checknameByName(bLabel.getLName(), bLabel.getBusinessId(), curUser.getCorpId());
        if (line > 0) {
            throw new ServiceException(ResultServiceEnums.NAME_IS_EXISTS);
        }
        UserUtil.setCurrentMergeOperation(bLabel);
        return new PersistModel(bLabelRepository.updateByPrimaryKeySelective(bLabel), MessageConstant.MESSAGE_ALERT_SUCCESS);
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, readOnly = false)
    public PersistModel removeLabel(String id) {
        int line = bLabelRepository.removeLabel(id);
        return new PersistModel(line, MessageConstant.MESSAGE_ALERT_SUCCESS);
    }
    /**
     * 查询时 为知识广场的课程 添加 标签
     * @param result
     * @param curUser
     * @return
     */
    public List<BLesson> addPushLable(List<BLesson> result ,CurUser curUser){
        result.forEach(lesson->{
            List<String> list = new ArrayList<>();
            String pushLable = bLessonRepository.getPushLable(lesson.getBusinessId());
            // 获取 推荐课程的 企业自定义 标签
            List<QueryLabelResDTO> lableList = new ArrayList<>();
            if(!"###".equals(lesson.getCorpId())){
                lableList = bLessonRepository.queryLabelByLessonId(lesson.getBusinessId());
            }else {
                lableList = bLessonRepository.queryLabelByLessonId2(lesson.getBusinessId(),curUser.getCorpId());
            }
            if(StringUtils.isNotEmpty(pushLable))
                list.addAll(asList(pushLable.split(",")));
            if( list != null && list.size() > 0){
                // 加入平台推荐 标签
                lableList.addAll(bLessonRepository.queryPushLabelByLessonId(list));
            }
            lesson.setLabelList(lableList);
        });
        return result;
    }

    /**
     * 生产分类中 new 的个数
     * @param resDTOS  一级分类带子分类集合
     * @return
     */
    public List<QueryCourseResDTO> addNewFolder(List<QueryCourseResDTO> resDTOS,List<BLesson> oldLessons){
        LogUtil.addnNewLog("-----------addNewFolder----------当前接口查询的一级分类个数---------"+resDTOS.size()+"--------------------");
        List<BLesson> lessons = new ArrayList<>();
        //筛选有分类的 new 课程
        for(BLesson lesson : oldLessons){
            if("1".equals(lesson.getIsNew()) && (lesson.getCourseId() != null || !"".equals(lesson.getCourseId())))
                lessons.add(lesson);
        }
        for (QueryCourseResDTO dto : resDTOS){  //一级分类
            int num = 0; //属于当前分类的课程数
            int secNum = 0 ;//下层分类中的课程数
            for (BLesson b : lessons){
                if(b.getCourseId().equals(dto.getKey()))
                    num++;
            }
            secNum+=num;//添加分类直属 new课程的数量
            if (dto.getChildren() != null){
                for (QueryCourseResDTO dto1 : dto.getChildren()){ //二级分类
                    int num1 = 0; //二级分类下直属的课程
                    int secNum1 = 0 ;//二级分类下 三级分类中的的 new课程数
                    for (BLesson b : lessons){
                        if(b.getCourseId().equals(dto1.getKey()))
                            num1++;
                    }
                    secNum1 += num1; //将二级分类中的 直属课 添加到一级分类中
                    if (dto.getChildren() != null){
                        for (QueryCourseResDTO dto2 : dto1.getChildren()){  //三级分类
                            int num2 = 0;
                            for (BLesson b : lessons){
                                if(b.getCourseId().equals(dto2.getKey()))
                                    num2++;
                            }
                            LogUtil.addnNewLog("-----------addNewFolder----------三级分类--"+dto2+"--的new个数-------"+num2+"--------------------");
                            dto2.setNewCount(num2);//将三级分类包含的 new课数量添加到 分类中
                            secNum1+=num2; //将三级分类中的 new课数量添加到 二级分类中
                        }
                    }
                    LogUtil.addnNewLog("-----------addNewFolder---------二级分类--"+dto1+"--的new个数-------"+secNum1+"--------------------");
                    dto1.setNewCount(secNum1);
                    secNum += secNum1;//将二级分类下三级分类中的数量添加到 二级分类中
                }
            }
            LogUtil.addnNewLog("-----------addNewFolder----------一级分类--"+dto+"--的new个数-------"+secNum+"--------------------");
            dto.setNewCount(secNum);
        }
        return resDTOS;
    }
}
